@@ -7,9 +7,10 @@ from __future__ import absolute_import, division, print_function
 import errno
 import os
 import sys
+import traceback
 
 from twisted.python import log, logfile, usage
-from twisted.python.compat import intToBytes
+from twisted.python.compat import (intToBytes, _bytesRepr, _PY3)
 from twisted.python.util import (
     switchUID, uidFromString, gidFromString, untilConcludes)
 from twisted.application import app, service
@@ -83,14 +84,15 @@ def checkPID(pidfile):
         return
     if os.path.exists(pidfile):
         try:
-            pid = int(open(pidfile).read())
+            with open(pidfile) as f:
+                pid = int(f.read())
         except ValueError:
             sys.exit('Pidfile %s contains non-numeric value' % pidfile)
         try:
             os.kill(pid, 0)
         except OSError as why:
-            if why[0] == errno.ESRCH:
-                # The pid doesn't exists.
+            if why.errno == errno.ESRCH:
+                # The pid doesn't exist.
                 log.msg('Removing stale pidfile %s' % pidfile, isError=True)
                 os.remove(pidfile)
             else:
@@ -197,6 +199,45 @@ class UnixApplicationRunner(app.ApplicationRunner):
         self.oldstderr = sys.stderr
 
 
+    def _formatChildException(self, exception):
+        """
+        Format the C{exception} in preparation for writing to the
+        status pipe.  This does the right thing on Python 2 if the
+        exception's message is Unicode, and in all cases limits the
+        length of the message afte* encoding to 100 bytes.
+
+        This means the returned message may be truncated in the middle
+        of a unicode escape.
+
+        @type exception: L{Exception}
+        @param exception: The exception to format.
+
+        @return: The formatted message, suitable for writing to the
+            status pipe.
+        @rtype: L{bytes}
+        """
+        # On Python 2 this will encode Unicode messages with the ascii
+        # codec and the backslashreplace error handler.
+        exceptionLine = traceback.format_exception_only(exception.__class__,
+                                                        exception)[-1]
+        # remove the trailing newline
+        formattedMessage = '1 %s' % exceptionLine.strip()
+        # On Python 3, encode the message the same way Python 2's
+        # format_exception_only does
+        if _PY3:
+            formattedMessage = formattedMessage.encode('ascii',
+                                                       'backslashreplace')
+        # By this point, the message has been encoded, if appropriate,
+        # with backslashreplace on both Python 2 and Python 3.
+        # Truncating the encoded message won't make it completely
+        # unreadable, and the reader should print out the repr of the
+        # message it receives anyway.  What it will do, however, is
+        # ensure that only 100 bytes are written to the status pipe,
+        # ensuring that the child doesn't block because the pipe's
+        # full.  This assumes PIPE_BUF > 100!
+        return formattedMessage[:100]
+
+
     def postApplication(self):
         """
         To be called after the application is created: start the application
@@ -208,16 +249,15 @@ class UnixApplicationRunner(app.ApplicationRunner):
         except Exception as ex:
             statusPipe = self.config.get("statusPipe", None)
             if statusPipe is not None:
-                # Limit the total length to the passed string to 100
-                strippedError = str(ex)[:98]
-                untilConcludes(os.write, statusPipe, "1 %s" % (strippedError,))
+                message = self._formatChildException(ex)
+                untilConcludes(os.write, statusPipe, message)
                 untilConcludes(os.close, statusPipe)
             self.removePID(self.config['pidfile'])
             raise
         else:
             statusPipe = self.config.get("statusPipe", None)
             if statusPipe is not None:
-                untilConcludes(os.write, statusPipe, "0")
+                untilConcludes(os.write, statusPipe, b"0")
                 untilConcludes(os.close, statusPipe)
         self.startReactor(None, self.oldstdout, self.oldstderr)
         self.removePID(self.config['pidfile'])
@@ -248,7 +288,7 @@ class UnixApplicationRunner(app.ApplicationRunner):
         """
         Set the filesystem root, the working directory, and daemonize.
 
-        @type chroot: C{str} or L{NoneType}
+        @type chroot: C{str} or L{None}
         @param chroot: If not None, a path to use as the filesystem root (using
             L{os.chroot}).
 
@@ -259,11 +299,11 @@ class UnixApplicationRunner(app.ApplicationRunner):
         @param nodaemon: A flag which, if set, indicates that daemonization
             should not be done.
 
-        @type umask: C{int} or L{NoneType}
+        @type umask: C{int} or L{None}
         @param umask: The value to which to change the process umask.
 
-        @type pidfile: C{str} or L{NoneType}
-        @param pidfile: If not C{None}, the path to a file into which to put
+        @type pidfile: C{str} or L{None}
+        @param pidfile: If not L{None}, the path to a file into which to put
             the PID of this process.
         """
         daemon = not nodaemon
@@ -341,9 +381,10 @@ class UnixApplicationRunner(app.ApplicationRunner):
         @rtype: C{int}
         """
         data = untilConcludes(os.read, readPipe, 100)
-        if data != "0":
-            msg = ("An error has occurred: '%s'\nPlease look at log "
-                   "file for more information.\n" % (data[2:],))
+        dataRepr = _bytesRepr(data[2:])
+        if data != b"0":
+            msg = ("An error has occurred: %s\nPlease look at log "
+                   "file for more information.\n" % (dataRepr,))
             untilConcludes(sys.__stderr__.write, msg)
             return 1
         return 0
@@ -357,11 +398,11 @@ class UnixApplicationRunner(app.ApplicationRunner):
         @param euid: A flag which, if set, indicates that only the I{effective}
             UID and GID should be set.
 
-        @type uid: C{int} or C{NoneType}
-        @param uid: If not C{None}, the UID to which to switch.
+        @type uid: C{int} or L{None}
+        @param uid: If not L{None}, the UID to which to switch.
 
-        @type gid: C{int} or C{NoneType}
-        @param gid: If not C{None}, the GID to which to switch.
+        @type gid: C{int} or L{None}
+        @param gid: If not L{None}, the GID to which to switch.
         """
         if uid is not None or gid is not None:
             extra = euid and 'e' or ''
